#include "object.h"
#include "message.h"

class FlOObjectKnot: public FlOObject {
 friend FlOObject* deserialize_knot(const XMLNode& x);
 private:
  class Location {
   public:
    std::string url_;
    Location(const std::string& u):url_(u) { };
    Location(const char* u):url_(u) { };
  };
  static FlOObject* deserialize(const XMLNode& x);
  /// Locations are physical URLs
  std::list<Location> locations_;
  /// Some attributes may be handled in a special way
 protected:
  FlOObjectKnot(const FlOObject& o):FlOObject(o) { };
 public:
  FlOObjectKnot(const std::string id);
  FlOObjectKnot(FlOService& service,const std::string id);

  virtual void serialize(XMLNode& x) const;
  virtual void serialize(XMLNode& x,FlOMessage& m) const;
  // static FlOObject* deserialize(const XMLNode& x,FlOMessage& m);
  // virtual void serialize(std::string& msg) const;
  // virtual void serialize(std::string& msg,FlOMessage& m) const;
  // static FlOObject* deserialize(const std::string& msg);
  // static FlOObject* deserialize(const std::string& msg,FlOMessage& m);
  virtual void notification(FlOMessage& msg,pFlOServiceRemote rsrv);
  virtual void maintain(void);
};

class FlOObjectKnotRef: public FlOObject {
 friend class FlOObjectKnot;
 private:
 protected:
  FlOObjectKnotRef(const FlOObject& o):FlOObject(o) { };
 public:
  FlOObjectKnotRef(const std::string id);
  FlOObjectKnotRef(FlOService& service,const std::string id);
};

//class FlOObjectGroup: public FlOObject {
// private:
// public:
//};

//class FlObjectGroupRef: public FlOObject {
// private:
// public:
//};

FlOObject* deserialize_knot(const XMLNode& x) {
  // check if knot
  XMLNode* x_kind = x.getNode("kind");
  if(x_kind == NULL) return NULL;
  std::string o_kind = x_kind->content();
  delete x_kind;
  if(o_kind != "knot") return NULL;
  return FlOObjectKnot::deserialize(x);
}

// support for ordinary and reference
FlOObject* FlOObjectKnot::deserialize(const XMLNode& x) {
  // parse attributes by deserializing into common-purpose object
  FlOObject* o = deserialize_object(x);
  if(o == NULL) return NULL;
  if(!(*o)) { delete o; return NULL; };
  if(o->type() == undefined) { delete o; return NULL; };
  if(o->type() == reference) {
    FlOObjectKnotRef* it = new FlOObjectKnotRef(*o);
    if(it == NULL) { delete o; return NULL; };
    if(!(*it)) { delete o; delete it; return NULL; };
    delete o;
    return it;
  };
  FlOObjectKnot* it = new FlOObjectKnot(o->id());
  if(it == NULL) { delete o; return NULL; };
  if(!(*it)) { delete o; delete it; return NULL; };
  delete o;
  // filter attributes to store 
  for(std::vector<FlOAttribute>::const_iterator i = o->attributes().begin();
                    i!=o->attributes().end();++i) {
    // Filter results
    if(i->name() == "location") {
      it->locations_.push_back(Location(i->value()));
    } else { 
      it->attrs_.push_back(*i);
    };
  };
  return it;
}

FlOObjectKnot::FlOObjectKnot(const std::string id):FlOObject(id,ordinary) {
  odlog(3)<<"FlOObjectKnot: constructor: id: "<<id<<endl;
}

FlOObjectKnot::FlOObjectKnot(FlOService& service,const std::string id):FlOObject(service,id,ordinary) {
  odlog(3)<<"FlOObjectKnot: constructor: id: "<<id<<endl;
}

FlOObjectKnotRef::FlOObjectKnotRef(const std::string id):FlOObject(id,reference) {
  odlog(3)<<"FlOObjectKnotRef: constructor: id: "<<id<<endl;
}

FlOObjectKnotRef::FlOObjectKnotRef(FlOService& service,const std::string id):FlOObject(service,id,reference) {
  odlog(3)<<"FlOObjectKnotRef: constructor: id: "<<id<<endl;
}

void FlOObjectKnot::serialize(XMLNode& x) const {
  FlOObject::serialize(x);
  // Add specific attributes
  XMLNode x_kind(x,"kind");
  x_kind.content("knot");
  for(std::list<Location>::const_iterator i = locations_.begin();
                                         i!=locations_.end();++i) {
    XMLNode x_attr(x,"attribute");
    XMLNode x_name(x_attr,"name");
    x_name.content("location");
    XMLNode x_value(x_attr,"value");
    x_value.content(i->url_);
  };
}

void FlOObjectKnot::serialize(XMLNode& x,FlOMessage& m) const {
  odlog(3)<<"FlOObjectKnot: serialize for message to xml"<<endl;
  serialize(x);
}

void FlOObjectKnot::notification(FlOMessage& msg,pFlOServiceRemote rsrv) {
  odlog(3)<<"FlOObjectKnot: notification"<<endl;
  switch(msg.type()) {
    case created: {
      odlog(3)<<"FlOObjectKnot: notification: created"<<endl;
      // Check if remotely created object has same attributes

    }; break;
    case updated: {
      odlog(3)<<"FlOObjectKnot: notification: updated"<<endl;
      // Analyze modification and accept/reject it

    }; break;
    case deleted: {
      odlog(3)<<"FlOObjectKnot: notification: deleted"<<endl;
      // Check number of data replicas left and initiate transfer if needed

    }; break;
  };
}

void FlOObjectKnot::maintain(void) {
  odlog(3)<<"FlOObjectKnot: maintain"<<endl;
  // Maitain proper number of replicas, etc.

}

