#ifndef XMLWRAP_H
#define XMLWRAP_H

#include <string>

#include <libxml/tree.h>

// class XMLDocument {
// public:
//  XMLDocument(void);
//};

/// Wrapper for sophisticated XML management API. 
/// Provides very simple, but also very simple to use functionality.
/// There is no separate XML document. Everything is node.
/// This implementation uses libxml2.
class XMLNode {
 private:
  struct _xmlNode* instance;
  bool allocated;
  XMLNode(struct _xmlNode* cur);
 public:
  /// Empty tree
  XMLNode(void):instance(NULL),allocated(false) { };
  /// Top named node
  XMLNode(const std::string& name);
  /// Copy contructor (copies whole subtree)
  XMLNode(const XMLNode& node);
  /// Named node will be attached to parent
  XMLNode(XMLNode& parent,const std::string& name);
  /// If node is not empty
  operator bool(void) const { return (instance != NULL); };
  /// If node is empty
  bool operator!(void) const { return (instance == NULL); };
  /// Replace this node with a copy of supplied one. This node
  /// becomes a top one.
  void operator=(const XMLNode& node);
  ~XMLNode(void);
  /// Attach copy of supplied node as a child
  void addNode(const XMLNode& node);
  /// Attach supplied node as a child (supplied node must be top node)
  void linkNode(XMLNode& node);
  /// Get pointer to child node by order number
  XMLNode* getNode(int n) const;
  /// Get pointer to child node by name (first found returned)
  XMLNode* getNode(const std::string& name) const;
  /// Get pointer to child node by name and order
  XMLNode* getNode(const std::string& name,int n) const;
  /// Get number of child nodes
  int nodes(void) const;
  /// Get name of this node
  std::string name(void) const;
  /// Set name of this node
  void name(const std::string& s);
  /// Get content of node (inluding all children)
  std::string content(void) const;
  /// Set content of this node
  void content(const std::string& s);
  /// Get XML subtree in printable format
  std::string get(void) const;
  /// Set new XML tree. Node becomes top node.
  void set(const std::string& s);
};

#endif // XMLWRAP_H
