package se.umu.jobsubmission.impl.broker; 

import org.apache.log4j.Logger; 
import org.ggf.schemas.jsdl._2005._11.jsdl.JobDefinition_Type;
import org.globus.gsi.GlobusCredential;

import se.umu.jobsubmission.impl.datamanager.DataHelper;
import se.umu.jobsubmission.JobPreferencesType;
import se.umu.jobsubmission.FileTransfer;
import se.umu.jobsubmission.SchedulingHintsType;


public class StageOutTimePredictor implements Predictor {

    private DataHelper manager; // Use Datamanager when we implement it...
    
    public static final int DEFAULT_STAGEOUT_TIME = 0; 
    private static Logger logger = Logger.getLogger(
    		StageOutTimePredictor.class); 
    
    /**
     * Predicts time required to stage output files
     * from a resource. For now, this is trivial as this
     * time is listed in the job preferences. 
     */
    public int predict(JobDefinition_Type jobDescription, 
    		Object resource, 
			JobPreferencesType preferences, 
			Object context, 
			GlobusCredential credential) { // throws Exception {
    	
    	try {
    		SchedulingHintsType hints = preferences.getSchedulingHints(); 
    		FileTransfer transfer = hints.getFileTransfers();
    		Integer stageOutTime = transfer.getStageOutTime(); 
    		return stageOutTime.intValue(); 
    	} catch (Exception e) {
    		logger.debug("Could not determine stage out time from " 
    				+ "job preferences."); 
    	}
    	
    	logger.debug("Using default stage out time: " 
    			+ StageOutTimePredictor.DEFAULT_STAGEOUT_TIME); 
    	return StageOutTimePredictor.DEFAULT_STAGEOUT_TIME; 
    	
    }

}
