package se.umu.jobsubmission.impl.dispatcher.test; 

import se.umu.jobsubmission.impl.dispatcher.ARCDispatcher; 

import org.globus.rsl.AbstractRslNode; 
import org.globus.rsl.RslNode; 
import org.globus.rsl.NameOpValue; 
import org.globus.rsl.RSLParser;

import org.apache.log4j.Logger;

import org.globus.mds.glue.GLUECERPType;


import org.globus.mds.glue.ClusterType; 
import org.globus.mds.glue.SubClusterType; 
import org.globus.mds.glue.SubClusterOrHostType; 
import org.globus.mds.glue.ArchitectureType; 
import org.globus.mds.glue.OperatingSystemType; 
import org.globus.mds.glue.ApplicationSoftwareType; 
import org.globus.mds.glue.FileSystemType; 
import org.globus.mds.glue.MainMemoryType; 
import org.globus.mds.glue.RunTimeEnvironmentType; 

import org.globus.mds.glue.ComputingElementType; 
import org.globus.mds.glue.InfoType; 
import org.globus.mds.glue.StateType; 
import org.globus.mds.glue.PolicyType; 


public class TestARCDispatcher {

    private static Logger logger = Logger.getLogger(TestARCDispatcher.class);


    // From ARCInformationConverter...
    public GLUECERPType createGLUECERP() {

        GLUECERPType GLUECERP = new GLUECERPType(); 

        // Cluster part

        // Can ARCResource hold >1 queue? In that case, this is too small...
        ClusterType clusters[] = new ClusterType[1]; 
        clusters[0] = new ClusterType(); 

        GLUECERP.setCluster(clusters); 

        SubClusterType subClusters[] = new SubClusterType[1]; 
        subClusters[0] = new SubClusterType(); 
        clusters[0].setSubCluster(subClusters); 

        ArchitectureType architecture = new ArchitectureType(); 
        subClusters[0].setArchitecture(architecture); 
        OperatingSystemType operatingSystem = new OperatingSystemType(); 
        subClusters[0].setOperatingSystem(operatingSystem); 
        ApplicationSoftwareType appSoftware = new ApplicationSoftwareType(); 
        subClusters[0].setApplicationSoftware(appSoftware); 
        FileSystemType fileSystems[] = new FileSystemType[1]; 
        fileSystems[0] = new FileSystemType(); 
        subClusters[0].setFileSystem(fileSystems); 
        MainMemoryType mainMemory = new MainMemoryType(); 
        subClusters[0].setMainMemory(mainMemory); 


        // ComputingElement part
        ComputingElementType computingElements[] = new ComputingElementType[1]; 
        computingElements[0] = new ComputingElementType(); 

        GLUECERP.setComputingElement(computingElements); 

        InfoType info = new InfoType(); 
        computingElements[0].setInfo(info); 
        StateType state = new StateType(); 
        computingElements[0].setState(state); 
        PolicyType policy = new PolicyType(); 
        computingElements[0].setPolicy(policy); 

        return GLUECERP; 

    }

    public void run(String []args) {

        try {
            if(args.length < 3) {
                System.err.println("Usage: TestARCDispatcher resourcename queuename xRSL"); 
                System.exit(1); 
            }

            String resourceURI = args[0]; 
            String queueName = args[1]; 
            String xrslString = args[2]; 

            RSLParser parser = new RSLParser(xrslString); 
            AbstractRslNode xrsl = parser.parse(); 

            logger.debug("Submitting job " + xrsl); 
            logger.debug("To resource: " + resourceURI); 
            logger.debug("In queue: " + queueName); 

            ARCDispatcher dispatcher = new ARCDispatcher(); 

            GLUECERPType jobContact= createGLUECERP();
            jobContact.getCluster(0).setUniqueID(resourceURI); 
            jobContact.getComputingElement(0).setName(queueName); 

            // Third arg is reservation identifier, 
            // currently used as "dumpxrsl" option
            String jobid = (String) dispatcher.requestJob(jobContact, xrsl, 
                    args.length == 3 ? new Object() : null); 

            System.out.println("Job submitted with jobid: " + jobid); 

        }
        catch (Exception e) {
            e.printStackTrace(); 
        }

    }


    public static void main(String []args) {

        TestARCDispatcher dispatcher = new TestARCDispatcher(); 
        dispatcher.run(args); 

    }
}
