package se.umu.jobsubmission.impl.informationfinder; 

import java.util.Calendar;

import org.apache.log4j.Logger;
import org.globus.gsi.GlobusCredential;

import se.umu.jobsubmission.util.Worker;

// Worker class running a ResourceFetcher
public class FetcherWorker extends Worker {

	private String fetcherClasses[]; 
    private int timeout; 
    
    private ResourceInformationCache resourceInformationCache; 
    
    private GlobusCredential credential; 

    private static Logger logger = Logger.getLogger(FetcherWorker.class); 

    public FetcherWorker(Object workUnit, 
    		String fetcherClasses[], 
			int timeout, 
			ResourceInformationCache cache, 
			GlobusCredential credential) {
        super(workUnit); 
        this.fetcherClasses = fetcherClasses; 
        this.timeout = timeout; 
        this.resourceInformationCache = cache; 
        this.credential = credential; 
    }

    public void doWork() {
        String resourceURI = (String) workUnit; 
        
        // 1: Check if info exists (and is valid) within cache
        CachedResourceInformation cachedInfo = 
        	this.resourceInformationCache.get(resourceURI); 
        if(cachedInfo != null) {    
        	logger.debug("Found cache entry for " + resourceURI); 
        	Calendar now = Calendar.getInstance(); 
        	Calendar expires = cachedInfo.getTimestamp(); 
        	if(expires.after(now)) {
        		logger.debug("Cache entry is valid for " + resourceURI); 
        		resultSet.add(cachedInfo.getInformation()); 
        		return; 
        	}
        	else {
        		logger.debug("Cached entry has expired, fetching new"); 
        	}
        } 
        
        logger.debug("Could not use cached information for: " + resourceURI); 
        
        // 2: If not, find it and add it to cache. 
        Object resourceInfo = null; 
        boolean done = false; 
        // Try each fetcher until one succeedes. 
        for(int i = 0; i < fetcherClasses.length && !done; i++) {
        	try {
        		InformationFetcher fetcher = (InformationFetcher) Class.forName(
        				fetcherClasses[i]).newInstance(); 
        		resourceInfo = fetcher.getInformation(resourceURI, 
        				this.timeout, 
						this.credential); 
        		done = true; 
        	} catch (Exception e) {
        		logger.info("No information retrieved from " 
        				+ resourceURI); 
        		resourceInfo = null; 
        	}
        }
        
        if(done = false) {
        	logger.info("Could not retrieve any information from " 
        			+ resourceURI); 
        	return; 
        }
        
        if(resourceInfo == null) 
            logger.info("User is not authorized to use" + resourceURI); 
        else {
            logger.debug("Adding information from " + resourceURI); 
            resultSet.add(resourceInfo); 
        }
    }
}

