/*
 * Created on Oct 14, 2005
 *
 */
package se.umu.jobsubmission.impl.informationfinder;

import java.net.InetSocketAddress;
import java.net.URL;
import java.util.Calendar;

import org.apache.axis.message.MessageElement;
import org.apache.log4j.Logger;
import org.globus.gsi.GlobusCredential;
import org.globus.mds.aggregator.types.GetResourcePropertyPollType;
import org.globus.mds.glue.ComputingElementType;
import org.globus.mds.glue.GLUECERPType;
import org.globus.mds.glue.StateType;
import org.globus.mds.glue.batchprovider.SchedulerType;
import org.globus.wsrf.container.ServiceContainer; 
import org.globus.wsrf.client.ServiceURL; 
import org.globus.wsrf.encoding.ObjectDeserializer;
import org.globus.wsrf.utils.AnyHelper;
import org.oasis.wsrf.properties.QueryResourcePropertiesResponse;
import org.oasis.wsrf.properties.QueryResourceProperties_Element;
import org.oasis.wsrf.properties.QueryResourceProperties_PortType;

import se.umu.jobsubmission.util.QueryUtils;

/**
 * @author tordsson
 * Based on GT4 wsrf-query client, used to retrieve information 
 * about a single GT4 resource. 
 * 
 * TODO: This must be rewritten once a correctly installed 
 * information system is in place at the toggle machines. 
 * 
 */
public class GT4InformationFetcher implements InformationFetcher {
	
	private static Logger logger = Logger.getLogger(GT4InformationFetcher.class); 
	
	// Test this one...
	public String translateURL(String url) {
		
		if(url.startsWith("http")) {
			return url; 
		}
		
		String newURL =  ServiceURL.getDefaultProtocol() 
			+ "://" 
			+ url 
			+ ":"
			+ ServiceContainer.DEFAULT_SECURE_PORT 
			+ "/wsrf/services/DefaultIndexService"; 
		
		return newURL; 
		
	}
	
	/**
	 * Adds some of the information from the schedulerObject to the
	 * Glue information (at appropiate locations). 
	 * @param glueInfo
	 * @param schedulerInfo
	 */
	public void updateGlue(GLUECERPType glueInfo, SchedulerType schedulerInfo) {
		
		ComputingElementType ce[] = glueInfo.getComputingElement(); 
		if(ce != null) {
			StateType state = ce[0].getState(); 
			
			if(state != null) {
				
				// totaljobs
				String totalJobs = schedulerInfo.getQueue(0).getTotalJobs().toString(); 
				if("unknown".equals(totalJobs)) {
					state.setTotalJobs(0); // Use some other value? 
				}
				else if("unlimited".equals(totalJobs)) {
					state.setTotalJobs(Integer.MAX_VALUE); 
				} else {
					state.setTotalJobs(new Integer(totalJobs).intValue()); 
				}
			} 
		} else {
			logger.debug("\"CE\" is null"); 
		}
		
		// Value of most of these attributes can be 
		// "17", "unlimited" or "unknown", which make 
		// them kind of messy to handle. 
		
	
		
		/* FIXME: 
		 This is the old and currently not supported format of SchedulerType 
		 <scheduler xmlns="http://mds.globus.org/batchproviders/2004/09" 
		 xmlns:ns12="http://mds.globus.org/batchproviders/2004/09">
		 <Info GRAMVersion="4.0.1" HostName="toggle01.hpc2n.umu.se" 
		 LRMSType="PBS" LRMSVersion="2.0.0p2" TotalCPUs="8"/>
		 
		 <Queue name="batch">
		 <totalnodes>1</totalnodes>
		 <freenodes>1</freenodes>
		 <maxCount>1</maxCount>
		 <maxTotalMemory>unlimited</maxTotalMemory>
		 <maxSingleMemory>unlimited</maxSingleMemory>
		 <maxCPUtime>unlimited</maxCPUtime>
		 <maxtime>7200</maxtime>
		 <maxReqNodes>unlimited</maxReqNodes>
		 <runningJobs>3</runningJobs>
		 <maxRunningJobs>unlimited</maxRunningJobs>
		 <maxJobsInQueue>unlimited</maxJobsInQueue>
		 <whenActive>unknown</whenActive>
		 <dispatchType>batch</dispatchType>
		 <totalJobs>3</totalJobs>
		 <status>enabled</status>
		 */
		
		// totalCPUs
		// This is already set in the state.freeCPUs, but 
		// the number is incorrect @ toggle
		// Furtermore, the schemas has changed and 
		// SchedulerType:Info is removed -> no way to access correct 
		// number of CPUs. 
		
		// freeCPUs
		// This is already set in the state.freeCPUs, but 
		// the number is incorrect @ toggle...
		
		// TODO: 
		// The following make up some rather useful policy info
		// and should be appended to GLUE:CE:Policy
		/*
	      <maxTotalMemory>unlimited</maxTotalMemory>
	      <maxSingleMemory>unlimited</maxSingleMemory>
	      <maxCPUtime>unlimited</maxCPUtime>
	      <maxtime>7200</maxtime>
	      <maxReqNodes>unlimited</maxReqNodes>
	      <maxRunningJobs>unlimited</maxRunningJobs>
	      <maxJobsInQueue>unlimited</maxJobsInQueue>
	      */ 
	}
	
	
	/**
	 * Tests if two host names actually refers to the same host. 
	 * Should also handle cases as
	 * "https://130.239.46.45:8443/wsrf/services/DefaultIndexService"
	 * being compared to "toggle02.hpc2n.umu.se" 
	 * @param host1
	 * @param host2
	 * @return
	 */
	public boolean sameHost(String host1, String host2) {		
				
		if(host1 == null) {
			return host2 == null; 
		}
		if(host2 == null) {
			return host1 == null; 
		}
		if(host1.equals(host2)) {
			return true; 
		}		
		// Extract host name part from URL and resolve IP addresses 
		try {
			String hostName1 = null; 
			String hostName2 = null; 
			if(host1.startsWith("http")) {
				URL url = new URL(host1); 
				String host = url.getHost(); 			
				hostName1 = new InetSocketAddress(host, 8443).getAddress().getHostName(); 							
				if(hostName1.equals(host2)) {
					return true; 
				}
			}
			if(host2.startsWith("http")) {
				URL url = new URL(host1); 
				String host = url.getHost(); 	
				hostName2 = new InetSocketAddress(host, 8443).getAddress().getHostName(); 							
				if(hostName2.equals(host1)) {
					return true; 
				}
			}
			if(hostName1 != null && hostName2 != null) {
				if(hostName1.equals(hostName2)) {
					return true; 
				}
			}
			
		} catch (Exception e) { 
			logger.warn("Host comparison between " 
					+ host1 + " and " + host2 + " failed: " + e.getMessage()); 
		}
		
		return false; 				
	}
	
	/**
	 * Send XPath query to DefaultIndexService running on host, 
	 * request GLUECE element from resource properties. 
	 */
	public Object getInformation(String resourceURL, 
			int timeout, 
			GlobusCredential credential) throws Exception {
		
		String serviceURL = null; 
		GLUECERPType glueInfo = null; 
		QueryResourceProperties_Element request = null; 
		QueryResourcePropertiesResponse response = null; 	
		QueryResourceProperties_PortType queryPortType = null; 
		
		logger.debug("Query to host: " + resourceURL); 		
		
		try {
			
			serviceURL = this.translateURL(resourceURL); 					
			
			logger.debug("After URL translation: " + serviceURL); 
			
			queryPortType = QueryUtils.getQueryPort(serviceURL,
				credential); 
			
			// Set timeout on stub: 
	        ((org.apache.axis.client.Stub)queryPortType).setTimeout(timeout); 
			
			// Build query for GLUE RP and send it to service. 
	       request = QueryUtils.getQuery("//*[local-name()='GLUECE']");
	       response = queryPortType.queryResourceProperties(request);	    
	       MessageElement any[] = response.get_any(); 
	       	       	       	       
	       // These are 3 entries, of which one lacks the CE element. 
	       // Actually, there are 3 entries for each cluster, 
	       // if the query is sent to a higher level indexservice 
	       // there will be more entries. 
	       
	       // Ensure that hostname part of the elements is the
	       // same as the resourceURL...
	       boolean done = false; 
	       for(int i = 0; i < any.length && !done; i++) {
	       	   glueInfo = (GLUECERPType) 
			   ObjectDeserializer.toObject(any[i], GLUECERPType.class);
	       	   ComputingElementType ce[] = null; 
	       	   String hostName = null; 
	       	   
	       	   // There are sometimes some partly incomplete GLUECE 
	       	   // elements in the retrieved information, skip these. 
	       	   try { 
	       	       ce = glueInfo.getComputingElement(); 	       	
	       	   	   hostName = glueInfo.getCluster(0).getSubCluster(0).getHost(0).getName(); 
	       	   } catch (Exception e) {
	       	   	   continue;
	       	   }
	       	
	       	   if(ce != null) {
	       	   	   if(sameHost(resourceURL, hostName)) { 
	       	   	   	  done = true; 
	       	   	   	  logger.debug("Found correct GLUECE entry"); 
	       	   	   }
	       	   	   else {
	       	   	   	   logger.debug("Resource host name " + hostName + 
	       	   	   	   		" does not match " + resourceURL); 
	       	   	   }
	       	   }
	       	   else {
	       	   	   logger.debug("Wrong GLUECE entry. lacking \"CE\"");
	       	   }
	       	   	   /*
	       	   	   StringWriter sw = new StringWriter(); 
	       	   	   ObjectSerializer.serialize(sw, glueInfo,
	       	   	   		GLUECERPType.getTypeDesc().getXmlType()); 
	       	   	   logger.warn("\n GLUECERP is: \n" + sw.toString());	        */ 
	       }
	       
	       if(!done) {
	       	    logger.info("No valid information found for " + serviceURL); 
	       	   throw new Exception("No valid information found for " + serviceURL); 
	       }
	       
	    	
		} catch (Exception e) {
		    logger.info("No cluster information retrieved form " + serviceURL); 	
		    throw new Exception("No cluster information retrieved form " 
		    		+ resourceURL); 
		}
	        
		try { 
			
	        // FIXME: When deserializing, SchedulerType is not the same type as 
	        // the one in the response from the DefaultIndexService 
	        // Some versioning problem? 
			
	        // Perform a second query, getting information about the 
	        // queue from the queueProvider 
	        request = QueryUtils.getQuery("//*[local-name()='scheduler']");
	        response = queryPortType.queryResourceProperties(request);
	        // System.out.println(AnyHelper.toSingleString(response));
	        MessageElement any[] = response.get_any(); 
	        	        
	        // TODO: Need to ensure that we get the correct entry here as well
	        // (test resourceURL.equals(hostName))...
                // --> But hostname is not deserialized correctly! 
	        
	        // Find correct entry 
	        SchedulerType schedulerInfo = null; 
	        boolean done = false; 
	        for(int i = 0; i < any.length && !done; i++) {
	        	
	        	schedulerInfo = (SchedulerType) ObjectDeserializer.toObject(
	        			any[0], SchedulerType.class); 	       
	        	
	        	String schedulerInfoAsString = AnyHelper.toString(any[0]); 
	        	
	        	/* 
	        	InfoType info = schedulerInfo.getInfo(); 
	        	String cpus = info.getTotalCPUs(); 
	        	String hostName = info.getHostName(); 
	        	logger.warn("For response " + i + " we got hostname " 
	        			+ hostName + " and " + cpus + " CPUs ");
	        	
	        	logger.warn("Sched info is of class: " + schedulerInfo.getClass()); 
	        	logger.warn("As string\n:" + schedulerInfoAsString); 
	        	
	        	
		        logger.warn("Whole scheduler info: "); 
		        StringWriter sw = new StringWriter(); 
	        	ObjectSerializer.serialize(sw, schedulerInfo,
	        			SchedulerType.getTypeDesc().getXmlType()); 	        		
	        	logger.debug(sw.toString());  */
	        	
	        	
	        }
	        
	        // Add some attributes currently only defined in the 
	        // schedulerInfo to appropiate places in glueInfo 
	        updateGlue(glueInfo, schedulerInfo); 	        	        
						
		} catch (Exception e) {
			// We at least got some information about the machine, 
			// let the broker operate using that
		    logger.error("No scheduler information retrieved from " + serviceURL); 
		}
		
		
		// Send a third query to get the poll interval
		try { 
						
	        request = QueryUtils.getQuery("//*[local-name()='GetResourcePropertyPollType']"); 
	        response = queryPortType.queryResourceProperties(request);
	        // System.out.println(AnyHelper.toSingleString(response));
	        
	        MessageElement any[] = response.get_any(); 
	        
	        int pollInterval = 0; 
	        if(any != null) {
	        	GetResourcePropertyPollType pollType = null; 
	        	boolean done = false; 
                        // Ensure that the timeout is for GLUE information and nothing else. 
	        	for(int i = 0; i < any.length && !done; i++) {	        			        	
	        		pollType = (GetResourcePropertyPollType) 
						ObjectDeserializer.toObject(any[i], GetResourcePropertyPollType.class);
	        		String pollTypeNamespace = pollType.getResourcePropertyName().getNamespaceURI();	        		
	        		String glueNamespace = ComputingElementType.getTypeDesc().getXmlType().getNamespaceURI(); 
	        		if(glueNamespace.equals(pollTypeNamespace)) {
	        			pollInterval = pollType.getPollIntervalMillis(); 
						done = true; 
	        		}
	        	}
	        } 
	        
	        if(pollInterval == 0) {
	        	logger.debug("Can not determine default poll interval, using 60s as interval"); 
	        	pollInterval = 60000; // TODO: Make this configurable... 
	        }
		
                // (Assume a pollinterval of 60 seconds) 
	        // Using poll interval as cache expiration time is 
	        // is somewhat misleading as it assumes as new data can be 
	        // generated anytime during these 60 seconds, i.e., the 
	        // data can be 55 seconds old and should expire in 5 seconds, not 
	        // in 60... 	        	        	        
                // With this in mind, let the cached entries be valid for a 
                // time x where x is randomly distributed between 0 and the interval: 
                // pollInterval = (int) ((Math.random() * pollInterval) + 500); 
                // On the other hand, in the long run, new data will be fetched
                // every 60 seconds, which is more correct than fetching 
                // new data every 60 * rand(), an expected value of 30 seconds. 
                //
                // -> Use pollInterval as it is, without random adjustments. 
                logger.debug("Cache validity time is: " + pollInterval + " ms"); 
	        
	        // Use GLUECERPType:any to store the cache validity time 
	        // as a string representing the time in ms (from 1/1 1970). 
	        Calendar now = Calendar.getInstance(); 
	        now.add(Calendar.SECOND, pollInterval / 1000); 
	        long timeInMillis = now.getTimeInMillis(); 
	        
	        MessageElement data = new MessageElement(
	        		new javax.xml.namespace.QName("http://www.w3.org/2001/XMLSchema", "long"), 
					"" + timeInMillis);	        
	        MessageElement[] elem = new MessageElement[]{data}; 
	        glueInfo.set_any(elem); 	        
		
		} catch (Exception e) {
			// Job submission can still be performed without 
			// poll interval. The only effect is that the next 
			// job not will be able to use cached information... 
			logger.error("Error getting information: " + e.getMessage());
		}	
		
		// Finally replace 'hostname' in glueinfo.cluster(0).subcluster(0).host(0).name
		// with https://hostname:port/wsrf/services/ManagedJobFactoryService 
		int index = serviceURL.lastIndexOf('/'); 	
		String gramURL = serviceURL.substring(0, index) + "/" + "ManagedJobFactoryService"; 
		glueInfo.getCluster(0).getSubCluster(0).getHost(0).setName(gramURL); 
		
		return glueInfo; 
	}
}
